@@ -11,6 +11,7 @@ log = logging.getLogger("app")
 #
 # APP
 #
+import FlaskSimpleAuth as fsa
 from FlaskSimpleAuth import Flask, jsonify, ANY, ALL, NONE, path, string
 app = Flask("Test")
 
@@ -31,7 +32,7 @@ app.config.update(
 
 # object permissions: dad (admin) or self
 def check_users_perms(login: str, val: str, mode):
-    return login in (val, "dad")
+    return login in (val, "dad") if val in UP else None
 
 app.register_object_perms("users", check_users_perms)
 
@@ -331,3 +332,8 @@ def get___def(_def: str, _int: int, _: Len, _pass: bool = True):
 @app.get("/my/<login>", authorize=("users", "login"))
 def get_my_login(login: str):
     return f"login is {login} for {app.get_user()}", 200
+
+# raise instead of return
+@app.get("/oops", authorize=ANY)
+def get_oops():
+    raise fsa.FSAException("Ooops!", 518)
